package com.os.ems.record.controller;

import java.util.List;
import javax.servlet.http.HttpServletResponse;
import org.springframework.security.access.prepost.PreAuthorize;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;
import com.os.common.annotation.Log;
import com.os.common.core.controller.BaseController;
import com.os.common.core.domain.AjaxResult;
import com.os.common.enums.BusinessType;
import com.os.ems.record.domain.EmsRecordAlarmData;
import com.os.ems.record.service.IEmsRecordAlarmDataService;
import com.os.common.utils.poi.ExcelUtil;
import com.os.common.core.page.TableDataInfo;

/**
 * 异常数据记录Controller
 * 
 * @author Yinq
 * @date 2024-05-15
 */
@RestController
@RequestMapping("/ems/record/recordAlarmData")
public class EmsRecordAlarmDataController extends BaseController
{
    @Autowired
    private IEmsRecordAlarmDataService emsRecordAlarmDataService;

    /**
     * 查询异常数据记录列表
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:list')")
    @GetMapping("/list")
    public TableDataInfo list(EmsRecordAlarmData emsRecordAlarmData)
    {
        startPage();
        List<EmsRecordAlarmData> list = emsRecordAlarmDataService.selectEmsRecordAlarmDataList(emsRecordAlarmData);
        return getDataTable(list);
    }

    /**
     * 导出异常数据记录列表
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:export')")
    @Log(title = "异常数据记录", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    public void export(HttpServletResponse response, EmsRecordAlarmData emsRecordAlarmData)
    {
        List<EmsRecordAlarmData> list = emsRecordAlarmDataService.selectEmsRecordAlarmDataList(emsRecordAlarmData);
        ExcelUtil<EmsRecordAlarmData> util = new ExcelUtil<EmsRecordAlarmData>(EmsRecordAlarmData.class);
        util.exportExcel(response, list, "异常数据记录数据");
    }

    /**
     * 获取异常数据记录详细信息
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:query')")
    @GetMapping(value = "/{objId}")
    public AjaxResult getInfo(@PathVariable("objId") Long objId)
    {
        return success(emsRecordAlarmDataService.selectEmsRecordAlarmDataByObjId(objId));
    }

    /**
     * 新增异常数据记录
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:add')")
    @Log(title = "异常数据记录", businessType = BusinessType.INSERT)
    @PostMapping
    public AjaxResult add(@RequestBody EmsRecordAlarmData emsRecordAlarmData)
    {
        emsRecordAlarmData.setCreateBy(getUsername());
        return toAjax(emsRecordAlarmDataService.insertEmsRecordAlarmData(emsRecordAlarmData));
    }

    /**
     * 修改异常数据记录
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:edit')")
    @Log(title = "异常数据记录", businessType = BusinessType.UPDATE)
    @PutMapping
    public AjaxResult edit(@RequestBody EmsRecordAlarmData emsRecordAlarmData)
    {
        emsRecordAlarmData.setUpdateBy(getUsername());
        return toAjax(emsRecordAlarmDataService.updateEmsRecordAlarmData(emsRecordAlarmData));
    }

    /**
     * 删除异常数据记录
     */
    @PreAuthorize("@ss.hasPermi('ems/record:recordAlarmData:remove')")
    @Log(title = "异常数据记录", businessType = BusinessType.DELETE)
	@DeleteMapping("/{objIds}")
    public AjaxResult remove(@PathVariable Long[] objIds)
    {
        return toAjax(emsRecordAlarmDataService.deleteEmsRecordAlarmDataByObjIds(objIds));
    }

    /**
     * 处理异常数据
     * @param objIds
     * @return
     */
    @PostMapping("/handleExceptions/{objIds}")
    public AjaxResult handleExceptions(@PathVariable Long[] objIds)
    {
        return toAjax(emsRecordAlarmDataService.handleExceptionsAlarmData(objIds));
    }

    /**
     * 获取异常数据表总数（包含所有告警类型）
     */
    @GetMapping("/getAlarmDataTotalCount")
    public int getAlarmDataTotalCount()
    {
        return emsRecordAlarmDataService.getAlarmDataTotalCount();
    }

    /**
     * 保存WebSocket告警数据（批量保存）
     * 前端传入多个EmsRecordAlarmData实体，每个告警规则对应一条记录
     * 
     * @param alarmDataList 异常数据记录列表
     * @return 结果
     */
    @Log(title = "WebSocket告警数据", businessType = BusinessType.INSERT)
    @PostMapping("/saveWebSocketAlarmData")
    public AjaxResult saveWebSocketAlarmData(@RequestBody List<EmsRecordAlarmData> alarmDataList)
    {
        if (alarmDataList == null || alarmDataList.isEmpty()) {
            return error("告警数据不能为空");
        }
        
        try {
            // 设置创建人信息
            String username = getUsername();
            for (EmsRecordAlarmData alarmData : alarmDataList) {
                alarmData.setCreateBy(username);
            }
            
            int result = emsRecordAlarmDataService.saveWebSocketAlarmDataBatch(alarmDataList);
            if (result > 0) {
                return success("保存成功，共保存 " + result + " 条告警记录");
            } else {
                return error("保存失败，没有新增任何记录");
            }
        } catch (Exception e) {
            logger.error("保存WebSocket告警数据异常", e);
            return error("保存失败：" + e.getMessage());
        }
    }

}
